/*****************************************************************************
 *
 *  Copyright (C) 2003 Cédric Brégardis <cedric.bregardis@free.fr>
 *
 *  This file is part of BRIQUOLO
 *
 *  BRIQUOLO is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  BRIQUOLO is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with BRIQUOLO; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/
#ifndef MOGL_TRIANGLEOMBRE
#define MOGL_TRIANGLEOMBRE

#include "MOGL_Structure.h"

/**
   C'est un triangle où seules les coordonnées dans l'espace sont précisées.
*/
class MOGL_TriangleOmbre
{
  protected :
        MOGL_Struct_Vecteur _Point1, _Point2, _Point3;
        MOGL_Struct_Vecteur _Normale;

  public :
        MOGL_TriangleOmbre(const MOGL_Struct_Vecteur & p_Point1,
                                           const MOGL_Struct_Vecteur & p_Point2,
                                           const MOGL_Struct_Vecteur & p_Point3);

        const MOGL_Struct_Vecteur & GetPoint(int NumPoint) const;

        const MOGL_Struct_Vecteur & GetNormale() const;
};

#endif
