/*****************************************************************************
 *
 *  Copyright (C) 2003 Cédric Brégardis <cedric.bregardis@free.fr>
 *
 *  This file is part of BRIQUOLO
 *
 *  BRIQUOLO is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  BRIQUOLO is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with BRIQUOLO; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/
#include "MOGL_Interface.h"

MOGL_Interface::MOGL_Interface(): _Niveau(0), _Cacher(false)
{
}

void MOGL_Interface::SetDimension(unsigned int p_Largeur, unsigned int p_Hauteur)
{
        _Largeur=p_Largeur;
        _Hauteur=p_Hauteur;
        _Rapport=static_cast<float>(p_Largeur) / static_cast<float>(p_Hauteur);
}

MOGL_Interface::~MOGL_Interface()
{
}

unsigned int MOGL_Interface::GetNiveau() const
{
  return _Niveau;
}

bool MOGL_Interface::GetCacher() const
{
  return _Cacher;
}

void MOGL_Interface::SetCacher(bool p_Cacher)
{
  _Cacher=p_Cacher;
}

